# Test helper file for example Train plugins

# This file's job is to collect any libraries needed for testing, as well as provide
# any utilities to make testing a plugin easier.

require "train/plugin_test_helper"
